@@ -5,47 +5,35 @@ files according to rfc2445.
 These are the defined components.
 
 """
-
+from __future__ import absolute_import
 import pytz
 from datetime import datetime
-from types import ListType, TupleType
-from icalendar.caselessdict import CaselessDict
-from icalendar.parser import (
+from .parser_tools import data_encode
+from .caselessdict import CaselessDict
+from .parser import (
     Contentlines,
     Contentline,
     Parameters,
-)
-from icalendar.parser import (
     q_split,
     q_join,
 )
-from icalendar.prop import (
-    TypesFactory,
-    vText,
-)
-
-SequenceTypes = (ListType, TupleType)
+from .prop import TypesFactory
+from .prop import vText, vDDDLists
+from .parser_tools import DEFAULT_ENCODING
 
 
 ######################################
 # The component factory
 
 class ComponentFactory(CaselessDict):
-    """ All components defined in rfc 2445 are registered in this factory
-    class. To get a component you can use it like this.
+    """All components defined in rfc 2445 are registered in this factory class.
+    To get a component you can use it like this.
 
-    >>> factory = ComponentFactory()
-    >>> component = factory['VEVENT']
-    >>> event = component(dtstart='19700101')
-    >>> event.to_ical()
-    'BEGIN:VEVENT\\r\\nDTSTART:19700101\\r\\nEND:VEVENT\\r\\n'
-
-    >>> factory.get('VCALENDAR', Component)
-    <class 'icalendar.cal.Calendar'>
     """
 
     def __init__(self, *args, **kwargs):
-        "Set keys to upper for initial dict"
+        """Set keys to upper for initial dict.
+        """
         CaselessDict.__init__(self, *args, **kwargs)
         self['VEVENT'] = Event
         self['VTODO'] = Todo
@@ -66,133 +54,12 @@ INLINE = CaselessDict(
 
 _marker = []
 
+
 class Component(CaselessDict):
-    """
-    Component is the base object for calendar, Event and the other components
-    defined in RFC 2445. normally you will not use this class directy, but
-    rather one of the subclasses.
+    """Component is the base object for calendar, Event and the other
+    components defined in RFC 2445. normally you will not use this class
+    directy, but rather one of the subclasses.
 
-    A component is like a dictionary with extra methods and attributes.
-    >>> c = Component()
-    >>> c.name = 'VCALENDAR'
-
-    Every key defines a property. A property can consist of either a single
-    item. This can be set with a single value
-    >>> c['prodid'] = '-//max m//icalendar.mxm.dk/'
-    >>> c
-    VCALENDAR({'PRODID': '-//max m//icalendar.mxm.dk/'})
-
-    or with a list
-    >>> c['ATTENDEE'] = ['Max M', 'Rasmussen']
-
-    if you use the add method you don't have to considder if a value is a list
-    or not.
-    >>> c = Component()
-    >>> c.name = 'VEVENT'
-    >>> c.add('attendee', 'maxm@mxm.dk')
-    >>> c.add('attendee', 'test@example.dk')
-    >>> c
-    VEVENT({'ATTENDEE': [vCalAddress('maxm@mxm.dk'), vCalAddress('test@example.dk')]})
-
-    You can get the values back directly
-    >>> c.add('prodid', '-//my product//')
-    >>> c['prodid']
-    vText(u'-//my product//')
-
-    or decoded to a python type
-    >>> c.decoded('prodid')
-    u'-//my product//'
-
-    With default values for non existing properties
-    >>> c.decoded('version', 'No Version')
-    'No Version'
-
-    The component can render itself in the RFC 2445 format.
-    >>> c = Component()
-    >>> c.name = 'VCALENDAR'
-    >>> c.add('attendee', 'Max M')
-    >>> c.to_ical()
-    'BEGIN:VCALENDAR\\r\\nATTENDEE:Max M\\r\\nEND:VCALENDAR\\r\\n'
-
-    >>> from icalendar.prop import vDatetime
-
-    Components can be nested, so You can add a subcompont. Eg a calendar holds events.
-    >>> e = Component(summary='A brief history of time')
-    >>> e.name = 'VEVENT'
-    >>> e.add('dtend', '20000102T000000', encode=0)
-    >>> e.add('dtstart', '20000101T000000', encode=0)
-    >>> e.to_ical()
-    'BEGIN:VEVENT\\r\\nDTEND:20000102T000000\\r\\nDTSTART:20000101T000000\\r\\nSUMMARY:A brief history of time\\r\\nEND:VEVENT\\r\\n'
-
-    >>> c.add_component(e)
-    >>> c.subcomponents
-    [VEVENT({'DTEND': '20000102T000000', 'DTSTART': '20000101T000000', 'SUMMARY': 'A brief history of time'})]
-
-    We can walk over nested componentes with the walk method.
-    >>> [i.name for i in c.walk()]
-    ['VCALENDAR', 'VEVENT']
-
-    We can also just walk over specific component types, by filtering them on
-    their name.
-    >>> [i.name for i in c.walk('VEVENT')]
-    ['VEVENT']
-
-    >>> [i['dtstart'] for i in c.walk('VEVENT')]
-    ['20000101T000000']
-
-    We can enumerate property items recursively with the property_items method.
-    >>> c.property_items()
-    [('BEGIN', 'VCALENDAR'), ('ATTENDEE', vCalAddress('Max M')), ('BEGIN', 'VEVENT'), ('DTEND', '20000102T000000'), ('DTSTART', '20000101T000000'), ('SUMMARY', 'A brief history of time'), ('END', 'VEVENT'), ('END', 'VCALENDAR')]
-
-    We can also enumerate property items just under the component.
-    >>> c.property_items(recursive=False)
-    [('BEGIN', 'VCALENDAR'), ('ATTENDEE', vCalAddress('Max M')), ('END', 'VCALENDAR')]
-    >>> sc = c.subcomponents[0]
-    >>> sc.property_items(recursive=False)
-    [('BEGIN', 'VEVENT'), ('DTEND', '20000102T000000'), ('DTSTART', '20000101T000000'), ('SUMMARY', 'A brief history of time'), ('END', 'VEVENT')]
-
-    Text fields which span multiple mulitple lines require proper indenting
-    >>> c = Calendar()
-    >>> c['description']=u'Paragraph one\\n\\nParagraph two'
-    >>> c.to_ical()
-    'BEGIN:VCALENDAR\\r\\nDESCRIPTION:Paragraph one\\\\n\\\\nParagraph two\\r\\nEND:VCALENDAR\\r\\n'
-
-    INLINE properties have their values on one property line. Note the double
-    quoting of the value with a colon in it.
-    >>> c = Calendar()
-    >>> c['resources'] = 'Chair, Table, "Room: 42"'
-    >>> c
-    VCALENDAR({'RESOURCES': 'Chair, Table, "Room: 42"'})
-
-    >>> c.to_ical()
-    'BEGIN:VCALENDAR\\r\\nRESOURCES:Chair\\\\, Table\\\\, "Room: 42"\\r\\nEND:VCALENDAR\\r\\n'
-
-    The inline values must be handled by the get_inline() and set_inline()
-    methods.
-
-    >>> c.get_inline('resources', decode=0)
-    ['Chair', 'Table', 'Room: 42']
-
-    These can also be decoded
-    >>> c.get_inline('resources', decode=1)
-    [u'Chair', u'Table', u'Room: 42']
-
-    You can set them directly
-    >>> c.set_inline('resources', ['A', 'List', 'of', 'some, recources'], encode=1)
-    >>> c['resources']
-    'A,List,of,"some, recources"'
-
-    and back again
-    >>> c.get_inline('resources', decode=0)
-    ['A', 'List', 'of', 'some, recources']
-
-    >>> c['freebusy'] = '19970308T160000Z/PT3H,19970308T200000Z/PT1H,19970308T230000Z/19970309T000000Z'
-    >>> c.get_inline('freebusy', decode=0)
-    ['19970308T160000Z/PT3H', '19970308T200000Z/PT1H', '19970308T230000Z/19970309T000000Z']
-
-    >>> freebusy = c.get_inline('freebusy', decode=1)
-    >>> type(freebusy[0][0]), type(freebusy[0][1])
-    (<type 'datetime.datetime'>, <type 'datetime.timedelta'>)
     """
 
     name = ''       # must be defined in each component
@@ -205,28 +72,27 @@ class Component(CaselessDict):
                                 # component, we will silently ignore
                                 # it, rather than let the exception
                                 # propagate upwards
+    # not_compliant = [''] # List of non-compliant properties.
 
     def __init__(self, *args, **kwargs):
-        "Set keys to upper for initial dict"
+        """Set keys to upper for initial dict.
+
+        """
         CaselessDict.__init__(self, *args, **kwargs)
         # set parameters here for properties that use non-default values
         self.subcomponents = [] # Components can be nested.
         self.is_broken = False  # True iff we ignored an exception while parsing a property
 
-#    def non_complience(self, warnings=0):
-#        """
-#        not implemented yet!
-#        Returns a dict describing non compliant properties, if any.
-#        If warnings is true it also returns warnings.
-#
-#        If the parser is too strict it might prevent parsing erroneous but
-#        otherwise compliant properties. So the parser is pretty lax, but it is
-#        possible to test for non-complience by calling this method.
-#        """
-#        nc = {}
-#        if not getattr(self, 'name', ''):
-#            nc['name'] = {'type':'ERROR', 'description':'Name is not defined'}
-#        return nc
+    #def is_compliant(self, name):
+    #    """Returns True is the given property name is compliant with the
+    #    icalendar implementation.
+    #
+    #    If the parser is too strict it might prevent parsing erroneous but
+    #    otherwise compliant properties. So the parser is pretty lax, but it is
+    #    possible to test for non-complience by calling this method.
+    #    """
+    #    return name in not_compliant
+
 
     #############################
     # handling of property values
@@ -235,7 +101,11 @@ class Component(CaselessDict):
         """Conditional convertion of values.
 
         """
-        if not cond: return value
+        if not cond:
+            return value
+        if type(value) in types_factory.all_types:
+            # Don't encode already encoded values.
+            return value
         klass = types_factory.for_property(name)
         obj = klass(value)
         if hasattr(value, 'params') and len(value.params.keys()) > 0:
@@ -244,7 +114,7 @@ class Component(CaselessDict):
         return obj
 
     def set(self, name, value, encode=1):
-        if encode and type(value) == ListType\
+        if encode and isinstance(value, list) \
                 and name.lower() not in ['rdate', 'exdate']:
             # Individually convert each value to an ical type except rdate and
             # exdate, where lists of dates might be passed to vDDDLists.
@@ -253,28 +123,7 @@ class Component(CaselessDict):
             self[name] = self._encode(name, value, encode)
 
     def add(self, name, value, encode=1):
-        """ Add a property.
-
-        Test the for timezone correctness: dtstart should preserve it's
-        timezone, crated, dtstamp and last-modified must be in UTC.
-        >>> from datetime import datetime
-        >>> import pytz
-        >>> from icalendar.cal import Component
-        >>> comp = Component()
-        >>> comp.add('dtstart', datetime(2010,10,10,10,0,0,tzinfo=pytz.timezone("Europe/Vienna")))
-        >>> comp.add('created', datetime(2010,10,10,12,0,0))
-        >>> comp.add('dtstamp', datetime(2010,10,10,14,0,0,tzinfo=pytz.timezone("Europe/Vienna")))
-        >>> comp.add('last-modified', datetime(2010,10,10,16,0,0,tzinfo=pytz.utc))
-
-        >>> lines = comp.to_ical().splitlines()
-        >>> "DTSTART;TZID=Europe/Vienna;VALUE=DATE-TIME:20101010T100000" in lines
-        True
-        >>> "CREATED;VALUE=DATE-TIME:20101010T120000Z" in lines
-        True
-        >>> "DTSTAMP;VALUE=DATE-TIME:20101010T130000Z" in lines
-        True
-        >>> "LAST-MODIFIED;VALUE=DATE-TIME:20101010T160000Z" in lines
-        True
+        """Add a property.
 
         """
         if isinstance(value, datetime) and\
@@ -290,7 +139,7 @@ class Component(CaselessDict):
         if name in self:
             oldval = self[name]
             value = self._encode(name, value, encode)
-            if type(oldval) == ListType:
+            if isinstance(oldval, list):
                 oldval.append(value)
             else:
                 self.set(name, [oldval, value], encode=0)
@@ -298,20 +147,39 @@ class Component(CaselessDict):
             self.set(name, value, encode)
 
     def _decode(self, name, value):
-        # internal for decoding property values
+        """Internal for decoding property values.
+
+        """
+
+        # TODO: Currently the decoded method calls the icalendar.prop instances
+        # from_ical. We probably want to decode properties into Python native
+        # types here. But when parsing from an ical string with from_ical, we
+        # want to encode the string into a real icalendar.prop property.
+        if isinstance(value, vDDDLists):
+            # TODO: Workaround unfinished decoding
+            return value
         decoded = types_factory.from_ical(name, value)
+        # TODO: remove when proper decoded is implemented in every prop.* class
+        # Workaround to decode vText properly
+        if isinstance(decoded, vText):
+            decoded = decoded.encode(DEFAULT_ENCODING)
         return decoded
 
     def decoded(self, name, default=_marker):
-        "Returns decoded value of property"
+        """Returns decoded value of property.
+
+        """
+        # XXX: fail. what's this function supposed to do in the end?
+        # -rnix
+
         if name in self:
             value = self[name]
-            if type(value) == ListType:
+            if isinstance(value, list):
                 return [self._decode(name, v) for v in value]
             return self._decode(name, value)
         else:
             if default is _marker:
-                raise KeyError, name
+                raise KeyError(name)
             else:
                 return default
 
@@ -320,34 +188,36 @@ class Component(CaselessDict):
     # property line. These methods are used for splitting and joining these.
 
     def get_inline(self, name, decode=1):
+        """Returns a list of values (split on comma).
+
         """
-        Returns a list of values (split on comma).
-        """
-        vals = [v.strip('" ').encode(vText.encoding)
-                  for v in q_split(self[name])]
+        vals = [v.strip('" ') for v in q_split(self[name])]
         if decode:
             return [self._decode(name, val) for val in vals]
         return vals
 
     def set_inline(self, name, values, encode=1):
-        """
-        Converts a list of values into comma seperated string and sets value to
-        that.
+        """Converts a list of values into comma seperated string and sets value
+        to that.
+
         """
         if encode:
             values = [self._encode(name, value, 1) for value in values]
-        joined = q_join(values).encode(vText.encoding)
-        self[name] = types_factory['inline'](joined)
+        self[name] = types_factory['inline'](q_join(values))
 
     #########################
     # Handling of components
 
     def add_component(self, component):
-        "add a subcomponent to this component"
+        """Add a subcomponent to this component.
+
+        """
         self.subcomponents.append(component)
 
     def _walk(self, name):
-        # private!
+        """Walk to given component.
+
+        """
         result = []
         if name is None or self.name == name:
             result.append(self)
@@ -356,9 +226,9 @@ class Component(CaselessDict):
         return result
 
     def walk(self, name=None):
-        """
-        Recursively traverses component and subcomponents. Returns sequence of
-        same. If name is passed, only components with name will be returned.
+        """Recursively traverses component and subcomponents. Returns sequence
+        of same. If name is passed, only components with name will be returned.
+
         """
         if not name is None:
             name = name.upper()
@@ -368,16 +238,16 @@ class Component(CaselessDict):
     # Generation
 
     def property_items(self, recursive=True):
-        """
-        Returns properties in this component and subcomponents as:
+        """Returns properties in this component and subcomponents as:
         [(name, value), ...]
+
         """
         vText = types_factory['text']
         properties = [('BEGIN', vText(self.name).to_ical())]
         property_names = self.sorted_keys()
         for name in property_names:
             values = self[name]
-            if type(values) == ListType:
+            if isinstance(values, list):
                 # normally one property is one line
                 for value in values:
                     properties.append((name, value))
@@ -390,22 +260,10 @@ class Component(CaselessDict):
         properties.append(('END', vText(self.name).to_ical()))
         return properties
 
-    @staticmethod
-    def from_ical(st, multiple=False):
-        """
-        Populates the component recursively from a string
+    @classmethod
+    def from_ical(cls, st, multiple=False):
+        """Populates the component recursively from a string.
 
-        RecurrenceIDs may contain a TZID parameter, if so, they should create
-        a tz localized datetime, otherwise, create a naive datetime
-        >>> componentStr = 'BEGIN:VEVENT\\nRECURRENCE-ID;TZID=America/Denver:20120404T073000\\nEND:VEVENT'
-        >>> component = Component.from_ical(componentStr)
-        >>> component['RECURRENCE-ID'].dt.tzinfo
-        <DstTzInfo 'America/Denver' MDT-1 day, 18:00:00 DST>
-
-        >>> componentStr = 'BEGIN:VEVENT\\nRECURRENCE-ID:20120404T073000\\nEND:VEVENT'
-        >>> component = Component.from_ical(componentStr)
-        >>> component['RECURRENCE-ID'].dt.tzinfo == None
-        True
         """
         stack = [] # a stack of components
         comps = []
@@ -419,7 +277,7 @@ class Component(CaselessDict):
                 # try and create one of the components defined in the spec,
                 # otherwise get a general Components for robustness.
                 c_name = vals.upper()
-                c_class = component_factory.get(c_name, Component)
+                c_class = component_factory.get(c_name, cls)
                 component = c_class()
                 if not getattr(component, 'name', ''): # undefined components
                     component.name = c_name
@@ -464,25 +322,27 @@ class Component(CaselessDict):
         return comps[0]
 
     def __repr__(self):
-        return '%s(' % self.name + dict.__repr__(self) + ')'
+        return '%s(%s)' % (self.name, data_encode(self))
 
-#    def content_line(self, name):
-#        "Returns property as content line"
-#        value = self[name]
-#        params = getattr(value, 'params', Parameters())
-#        return Contentline.from_parts((name, params, value))
+    def content_line(self, name, value):
+        """Returns property as content line.
+        """
+        params = getattr(value, 'params', Parameters())
+        return Contentline.from_parts(name, params, value)
 
     def content_lines(self):
-        "Converts the Component and subcomponents into content lines"
+        """Converts the Component and subcomponents into content lines.
+        """
         contentlines = Contentlines()
-        for name, values in self.property_items():
-            params = getattr(values, 'params', Parameters())
-            contentlines.append(Contentline.from_parts((name, params, values)))
+        for name, value in self.property_items():
+            cl = self.content_line(name, value)
+            contentlines.append(cl)
         contentlines.append('') # remember the empty string in the end
         return contentlines
 
     def to_ical(self):
-        return self.content_lines().to_ical()
+        content_lines = self.content_lines()
+        return content_lines.to_ical()
 
 
 #######################################
@@ -507,7 +367,7 @@ class Event(Component):
     )
     exclusive = ('DTEND', 'DURATION', )
     multiple = (
-        'ATTACH', 'ATTENDEE', 'CATEGORIES', 'COMMENT','CONTACT', 'EXDATE',
+        'ATTACH', 'ATTENDEE', 'CATEGORIES', 'COMMENT', 'CONTACT', 'EXDATE',
         'EXRULE', 'RSTATUS', 'RELATED', 'RESOURCES', 'RDATE', 'RRULE'
     )
     ignore_exceptions = True
@@ -590,59 +450,9 @@ class Alarm(Component):
 
 
 class Calendar(Component):
-    """
-    This is the base object for an iCalendar file.
-
-    Setting up a minimal calendar component looks like this
-    >>> cal = Calendar()
-
-    Som properties are required to be compliant
-    >>> cal['prodid'] = '-//My calendar product//mxm.dk//'
-    >>> cal['version'] = '2.0'
-
-    We also need at least one subcomponent for a calendar to be compliant
-    >>> from datetime import datetime
-    >>> event = Event()
-    >>> event['summary'] = 'Python meeting about calendaring'
-    >>> event['uid'] = '42'
-    >>> event.set('dtstart', datetime(2005,4,4,8,0,0))
-    >>> cal.add_component(event)
-    >>> cal.subcomponents[0].to_ical()
-    'BEGIN:VEVENT\\r\\nSUMMARY:Python meeting about calendaring\\r\\nDTSTART;VALUE=DATE-TIME:20050404T080000\\r\\nUID:42\\r\\nEND:VEVENT\\r\\n'
-
-    Write to disc
-    >>> import tempfile, os
-    >>> directory = tempfile.mkdtemp()
-    >>> open(os.path.join(directory, 'test.ics'), 'wb').write(cal.to_ical())
-
-    Parsing a complete calendar from a string will silently ignore bogus events.
-    The bogosity in the following is the third EXDATE: it has an empty DATE.
-    >>> s = '\\r\\n'.join(('BEGIN:VCALENDAR',
-    ...                    'PRODID:-//Google Inc//Google Calendar 70.9054//EN',
-    ...                    'VERSION:2.0',
-    ...                    'CALSCALE:GREGORIAN',
-    ...                    'METHOD:PUBLISH',
-    ...                    'BEGIN:VEVENT',
-    ...                    'DESCRIPTION:Perfectly OK event',
-    ...                    'DTSTART;VALUE=DATE:20080303',
-    ...                    'DTEND;VALUE=DATE:20080304',
-    ...                    'RRULE:FREQ=DAILY;UNTIL=20080323T235959Z',
-    ...                    'EXDATE;VALUE=DATE:20080311',
-    ...                    'END:VEVENT',
-    ...                    'BEGIN:VEVENT',
-    ...                    'DESCRIPTION:Bogus event',
-    ...                    'DTSTART;VALUE=DATE:20080303',
-    ...                    'DTEND;VALUE=DATE:20080304',
-    ...                    'RRULE:FREQ=DAILY;UNTIL=20080323T235959Z',
-    ...                    'EXDATE;VALUE=DATE:20080311',
-    ...                    'EXDATE;VALUE=DATE:',
-    ...                    'END:VEVENT',
-    ...                    'END:VCALENDAR'))
-    >>> [e['DESCRIPTION'].to_ical() for e in Calendar.from_ical(s).walk('VEVENT')]
-    ['Perfectly OK event']
+    """This is the base object for an iCalendar file.
 
     """
-
     name = 'VCALENDAR'
     canonical_order = ('VERSION', 'PRODID', 'CALSCALE', 'METHOD',)
     required = ('prodid', 'version', )
